==> org-lsst-ccs-ui-3.8.0-sources.jar: org/lsst/ccs/gconsole/plugins/commandbrowser/Browser.java <==
package org.lsst.ccs.gconsole.plugins.commandbrowser;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.Serializable;
import java.util.*;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.bus.states.CommandState;
import org.lsst.ccs.bus.states.ConfigurationState;
import org.lsst.ccs.bus.states.OperationalState;
import org.lsst.ccs.bus.states.PhaseState;
import org.lsst.ccs.bus.states.StateBundle;
import org.lsst.ccs.command.DictionaryCommand;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.gconsole.base.Console;
import org.lsst.ccs.gconsole.base.filter.CoreStates;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusAggregator;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusEvent;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusListener;
import org.lsst.ccs.gconsole.services.lock.Locker;
import org.lsst.ccs.gconsole.services.lock.LockService;
import org.lsst.ccs.gconsole.services.persist.DataPanelDescriptor;
import org.lsst.ccs.gconsole.services.persist.Savable;
import org.lsst.ccs.utilities.logging.Logger;

/**
 * Interface to be implemented by all types of command browsers.
 * A {@code Browser} handles the root page of the command browser tool.
 * All access to implementations of this class should happen on EDT.
 *
 * @author onoprien
 */
public abstract class Browser implements Savable {

// -- Fields : -----------------------------------------------------------------
    
    protected final Logger logger = Console.getConsole().getLogger();
    protected final LockService service = LockService.getService();
    
    protected JPanel browserPanel; // non-null if initialized
        
    ImageIcon ICON_UNLOCKED_BLACK = new ImageIcon(Browser.class.getResource("ic_lock_open_black_18dp.png"), "unlocked black");
    ImageIcon ICON_LOCKED_BLACK = new ImageIcon(Browser.class.getResource("ic_lock_black_18dp.png"), "locked black");
    ImageIcon ICON_UNLOCKED_RED = new ImageIcon(Browser.class.getResource("red_unlocked_27.png"), "unlocked red");
    ImageIcon ICON_LOCKED_RED = new ImageIcon(Browser.class.getResource("red_locked_27.png"), "locked red");
    ImageIcon ICON_UNLOCKED_GREEN = new ImageIcon(Browser.class.getResource("green_unlocked_27.png"), "unlocked green");
    ImageIcon ICON_LOCKED_GREEN = new ImageIcon(Browser.class.getResource("green_locked_27.png"), "locked green");
    ImageIcon ICON_UNLOCKED_GRAY = new ImageIcon(Browser.class.getResource("gray_unlocked_27.png"), "unlocked gray");
    ImageIcon ICON_LOCKED_GRAY = new ImageIcon(Browser.class.getResource("gray_locked_27.png"), "locked gray");
    
    protected JLabel stateLabel;
    private AgentStatusListener statusListener; // updates agent state display
    private final AgentStatusAggregator statusAggregator = Console.getConsole().getStatusAggregator();
    
    
// -- Life cycle : -------------------------------------------------------------
    
    /**
     * Initializes the browser if necessary, and returns its root graphics component.
     * @return Root graphics component for this browser.
     */
    public JComponent getPanel() {
        
        stateLabel = new JLabel(" ");
        statusListener = new AgentStatusListener() {
            @Override
            public void connect(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
            @Override
            public void disconnect(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
            @Override
            public void statusChanged(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
        };
        String agentName = getAgentName();
        List<String> agents = agentName == null ? null : Collections.singletonList(agentName);
        statusAggregator.addListener(statusListener, agents, CoreStates.CHANNELS);
        
        browserPanel = new JPanel(new BorderLayout());
        return browserPanel;
    }
    
    /**
     * Releases resources and shuts down this browser.
     */
    public void shutdown() {
        statusAggregator.removeListener(statusListener);
        browserPanel = null;
    }
    
    
// -- Getters : ----------------------------------------------------------------
    
    /**
     * Returns the name of this command browser.
     * @return Browser name.
     */
    abstract String getName();

    /**
     * Returns the name of the currently selected agent.
     * 
     * @return Current agent name, or {@code null} if this browser does not currently associated with a specific agent.
     */
    abstract String getAgentName();
    
    
// -- Utilities : --------------------------------------------------------------
    
    /**
     * Returns {@code true} if the specified command is currently unavailable for use based on lock and level settings.
     * 
     * @param command Command to check.
     * @param agent Lock service proxy for a remote agent.
     * @return {@code true} If the command is locked or {@code agent} is {@code null}.
     */
    static public boolean isCommandLocked(DictionaryCommand command, Locker agent) {
        if (agent == null) {
            return true;
        } else {
            return ((command.getLevel() > agent.getLevel()) || 
                    (agent.getState() != LockService.State.ATTACHED && command.getType() != CommandType.QUERY));
        }
    }
    
    protected void updateStateDisplay() {
        String agentName = getAgentName();
        if (agentName == null) {
            stateLabel.setText(" ");
            return;
        }
        StateBundle state = statusAggregator.getAgentState(agentName);
        if (state == null) {
            stateLabel.setText("OFFLINE");
            return;
        }
        StringBuilder sb = new StringBuilder("<html>");
        PhaseState phase = state.getState(PhaseState.class);
        if (phase != null) sb.append(phase).append(" - ");
        OperationalState ops = state.getState(OperationalState.class);
        if (ops != null) {
            switch (ops) {
                case NORMAL:
                    sb.append("<font color=\"#008800\">NORMAL</font>"); break;
                case ENGINEERING_FAULT:
                    sb.append("<font color=\"#DD0000\">ENGINEERING_FAULT</font>"); break;
                default:
                    sb.append(ops);
            }
            sb.append(" - ");
        }
        ConfigurationState con = state.getState(ConfigurationState.class);
        if (con != null) {
            switch (con) {
                case DIRTY:
                    sb.append("<font color=\"#0000DD\">MODIFIED CONFIGURATION</font>"); break;
                default:
                    sb.append(con);
            }
            sb.append(" - ");
        }
        AlertState als = state.getState(AlertState.class);
        if (als != null && !als.equals(AlertState.NOMINAL)) {
            switch (als) {
                case WARNING:
                    sb.append("<font color=\"#0000DD\">WARNING</font>"); break;
                case ALARM:
                    sb.append("<font color=\"#DD0000\">ALARM</font>"); break;
            }
            sb.append(" - ");
        }
        CommandState com = state.getState(CommandState.class);
        if (com != null) {
            switch (com) {
                case READY:
                    sb.append("<font color=\"#008800\">READY</font>"); break;
                case ACTIVE:
                    sb.append("<font color=\"#DD0000\">ACTIVE</font>"); break;
            }
        }
        stateLabel.setText(sb.toString());
    }
    
    protected Action getAction(String agentName, LockService.Operation operation) {
        Action act = new AbstractAction(operation.toString()) {
            @Override
            public void actionPerformed(ActionEvent e) {
                service.executeOperation(operation, agentName, 0);
            }
        };
        act.putValue(Action.SHORT_DESCRIPTION, operation +" "+ agentName);
        return act;
    }
    
    /**
     * Computes information for rendering lock icon and enabling actions associated with the specified agent.
     * @param agent Remote agent.
     * @return Rendered lock parameters.
     */
    protected RenderedLock renderLock(Locker agent) {
        RenderedLock out = new RenderedLock();
        if (agent == null) {
            out.icon = ICON_UNLOCKED_GRAY;
            out.tooltip = "Subsystem offline, unlocked";
            out.enabled = true;
        } else {
            out.enabled = !agent.isAdjusting();
            switch (agent.getState()) {
                case UNLOCKED:
                    out.icon = ICON_UNLOCKED_BLACK;
                    out.tooltip = "Double-click lo lock";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.LOCK);
                    out.availableActions = Collections.singletonList(out.defaultAction);
                    break;
                case LOCKED:
                    out.icon = ICON_LOCKED_BLACK;
                    out.tooltip = "Locked by "+ agent.getLock().getOwner();
                    out.defaultAction = null;
                    out.availableActions = Collections.singletonList(getAction(agent.getName(), LockService.Operation.DESTROY));
                    break;
                case DETACHED:
                    out.icon = ICON_LOCKED_RED;
                    out.tooltip = "Locked, double-click to attach";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.ATTACH);
                    out.availableActions = Arrays.asList(new Action[]{
                        getAction(agent.getName(), LockService.Operation.UNLOCK),
                        out.defaultAction
                    });
                    break;
                case ATTACHED:
                    out.icon = ICON_LOCKED_GREEN;
                    out.tooltip = "Locked, attached, double-click to detach";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.DETACH);
                    out.availableActions = Arrays.asList(new Action[]{
                        getAction(agent.getName(), LockService.Operation.UNLOCK),
                        out.defaultAction
                    });
                    break;
            }
            if (!out.enabled) {
                out.tooltip = "Waiting for response from Lock Manager";
                out.defaultAction = null;
                out.availableActions = null;
            }
        }        
        return out;
    }
    
    
    static protected class RenderedLock {
        public ImageIcon icon;
        public String tooltip;
        public boolean enabled;
        public Action defaultAction;
        public List<Action> availableActions;
        public RenderedLock(ImageIcon icon, String tooltip, boolean enabled, Action defaultAction, List<Action> availableActions) {
            this.icon = icon;
            this.tooltip = tooltip;
            this.enabled = enabled;
            this.defaultAction = defaultAction;
            this.availableActions = availableActions;
        }
        public RenderedLock() {
        }
    }

// -- Savin/restoring : --------------------------------------------------------

    @Override
    abstract public Descriptor save();
    
    static public class Descriptor implements Serializable {

        private String name;
        private String agent;
        private AgentPanel.Descriptor agentPanel;
        private DataPanelDescriptor page;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getAgent() {
            return agent;
        }

        public void setAgent(String agent) {
            this.agent = agent;
        }

        public AgentPanel.Descriptor getAgentPanel() {
            return agentPanel;
        }

        public void setAgentPanel(AgentPanel.Descriptor agentPanel) {
            this.agentPanel = agentPanel;
        }

        public DataPanelDescriptor getPage() {
            return page;
        }

        public void setPage(DataPanelDescriptor page) {
            this.page = page;
        }

    }
    
}

==> org-lsst-ccs-ui-4.2.0-sources.jar: org/lsst/ccs/gconsole/plugins/commandbrowser/Browser.java <==
package org.lsst.ccs.gconsole.plugins.commandbrowser;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.Serializable;
import java.util.*;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.lsst.ccs.bus.states.AlertState;
import org.lsst.ccs.bus.states.CommandState;
import org.lsst.ccs.bus.states.ConfigurationState;
import org.lsst.ccs.bus.states.OperationalState;
import org.lsst.ccs.bus.states.PhaseState;
import org.lsst.ccs.bus.states.StateBundle;
import org.lsst.ccs.command.DictionaryCommand;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.gconsole.base.Console;
import org.lsst.ccs.gconsole.base.filter.CoreStates;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusAggregator;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusEvent;
import org.lsst.ccs.gconsole.services.aggregator.AgentStatusListener;
import org.lsst.ccs.gconsole.services.lock.Locker;
import org.lsst.ccs.gconsole.services.lock.LockService;
import org.lsst.ccs.gconsole.services.persist.DataPanelDescriptor;
import org.lsst.ccs.gconsole.services.persist.Savable;
import org.lsst.ccs.utilities.logging.Logger;

/**
 * Interface to be implemented by all types of command browsers.
 * A {@code Browser} handles the root page of the command browser tool.
 * All access to implementations of this class should happen on EDT.
 *
 * @author onoprien
 */
public abstract class Browser implements Savable {

// -- Fields : -----------------------------------------------------------------
    
    protected final Logger logger = Console.getConsole().getLogger();
    protected final LockService service = LockService.getService();
    
    protected JPanel browserPanel; // non-null if initialized
        
    ImageIcon ICON_UNLOCKED_BLACK = new ImageIcon(Browser.class.getResource("ic_lock_open_black_18dp.png"), "unlocked black");
    ImageIcon ICON_LOCKED_BLACK = new ImageIcon(Browser.class.getResource("ic_lock_black_18dp.png"), "locked black");
    ImageIcon ICON_LOCKED_RED = new ImageIcon(Browser.class.getResource("red_locked_27.png"), "locked red");
    ImageIcon ICON_LOCKED_GREEN = new ImageIcon(Browser.class.getResource("green_locked_27.png"), "locked green");
    ImageIcon ICON_UNLOCKED_GRAY = new ImageIcon(Browser.class.getResource("gray_unlocked_27.png"), "unlocked gray");
    ImageIcon ICON_LOCKED_GRAY = new ImageIcon(Browser.class.getResource("gray_locked_27.png"), "locked gray");
    
    protected JLabel stateLabel;
    private AgentStatusListener statusListener; // updates agent state display
    private final AgentStatusAggregator statusAggregator = Console.getConsole().getStatusAggregator();
    
    
// -- Life cycle : -------------------------------------------------------------
    
    /**
     * Initializes the browser if necessary, and returns its root graphics component.
     * @return Root graphics component for this browser.
     */
    public JComponent getPanel() {
        
        stateLabel = new JLabel(" ");
        statusListener = new AgentStatusListener() {
            @Override
            public void connect(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
            @Override
            public void disconnect(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
            @Override
            public void statusChanged(AgentStatusEvent event) {
                SwingUtilities.invokeLater(() -> {
                    updateStateDisplay();
                });
            }
        };
        String agentName = getAgentName();
        List<String> agents = agentName == null ? null : Collections.singletonList(agentName);
        statusAggregator.addListener(statusListener, agents, CoreStates.CHANNELS);
        
        browserPanel = new JPanel(new BorderLayout());
        return browserPanel;
    }
    
    /**
     * Releases resources and shuts down this browser.
     */
    public void shutdown() {
        statusAggregator.removeListener(statusListener);
        browserPanel = null;
    }
    
    
// -- Getters : ----------------------------------------------------------------
    
    /**
     * Returns the name of this command browser.
     * @return Browser name.
     */
    abstract String getName();

    /**
     * Returns the name of the currently selected agent.
     * 
     * @return Current agent name, or {@code null} if this browser does not currently associated with a specific agent.
     */
    abstract String getAgentName();
    
    
// -- Utilities : --------------------------------------------------------------
    
    /**
     * Returns {@code true} if the specified command is currently unavailable for use based on lock and level settings.
     * 
     * @param command Command to check.
     * @param agent Lock service proxy for a remote agent.
     * @return {@code true} If the command is locked or {@code agent} is {@code null}.
     */
    static public boolean isCommandLocked(DictionaryCommand command, Locker agent) {
        if (agent == null) {
            return true;
        } else {
            return ((command.getLevel() > agent.getLevel()) || 
                    (agent.getState() != LockService.State.ATTACHED && command.getType() != CommandType.QUERY));
        }
    }
    
    protected void updateStateDisplay() {
        String agentName = getAgentName();
        if (agentName == null) {
            stateLabel.setText(" ");
            return;
        }
        StateBundle state = statusAggregator.getAgentState(agentName);
        if (state == null) {
            stateLabel.setText("OFFLINE");
            return;
        }
        StringBuilder sb = new StringBuilder("<html>");
        PhaseState phase = state.getState(PhaseState.class);
        if (phase != null) sb.append(phase).append(" - ");
        OperationalState ops = state.getState(OperationalState.class);
        if (ops != null) {
            switch (ops) {
                case NORMAL:
                    sb.append("<font color=\"#008800\">NORMAL</font>"); break;
                case ENGINEERING_FAULT:
                    sb.append("<font color=\"#DD0000\">ENGINEERING_FAULT</font>"); break;
                default:
                    sb.append(ops);
            }
            sb.append(" - ");
        }
        ConfigurationState con = state.getState(ConfigurationState.class);
        if (con != null) {
            switch (con) {
                case DIRTY:
                    sb.append("<font color=\"#0000DD\">MODIFIED CONFIGURATION</font>"); break;
                default:
                    sb.append(con);
            }
            sb.append(" - ");
        }
        AlertState als = state.getState(AlertState.class);
        if (als != null && !als.equals(AlertState.NOMINAL)) {
            switch (als) {
                case WARNING:
                    sb.append("<font color=\"#0000DD\">WARNING</font>"); break;
                case ALARM:
                    sb.append("<font color=\"#DD0000\">ALARM</font>"); break;
            }
            sb.append(" - ");
        }
        CommandState com = state.getState(CommandState.class);
        if (com != null) {
            switch (com) {
                case READY:
                    sb.append("<font color=\"#008800\">READY</font>"); break;
                case ACTIVE:
                    sb.append("<font color=\"#DD0000\">ACTIVE</font>"); break;
            }
        }
        stateLabel.setText(sb.toString());
    }
    
    protected Action getAction(String agentName, LockService.Operation operation) {
        Action act = new AbstractAction(operation.toString()) {
            @Override
            public void actionPerformed(ActionEvent e) {
                service.executeOperation(operation, agentName, 0);
            }
        };
        act.putValue(Action.SHORT_DESCRIPTION, operation +" "+ agentName);
        return act;
    }
    
    /**
     * Computes information for rendering lock icon and enabling actions associated with the specified agent.
     * @param agent Remote agent.
     * @return Rendered lock parameters.
     */
    protected RenderedLock renderLock(Locker agent) {
        RenderedLock out = new RenderedLock();
        if (agent == null) {
            out.icon = ICON_UNLOCKED_GRAY;
            out.tooltip = "Subsystem offline, unlocked";
            out.enabled = true;
            out.sortKey = "9";
        } else {
            out.enabled = !agent.isAdjusting();
            switch (agent.getState()) {
                case UNLOCKED:
                    out.icon = ICON_UNLOCKED_BLACK;
                    out.tooltip = "Double-click lo lock";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.LOCK);
                    out.availableActions = Collections.singletonList(out.defaultAction);
                    out.sortKey = "8"+ agent.getName();
                    break;
                case LOCKED:
                    out.icon = ICON_LOCKED_BLACK;
                    out.tooltip = "Locked by "+ agent.getLock().getOwner();
                    out.defaultAction = null;
                    out.availableActions = Collections.singletonList(getAction(agent.getName(), LockService.Operation.DESTROY));
                    out.sortKey = "7"+ agent.getName();
                    break;
                case DETACHED:
                    out.icon = ICON_LOCKED_RED;
                    out.tooltip = "Locked, double-click to attach";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.ATTACH);
                    out.availableActions = Arrays.asList(new Action[]{
                        getAction(agent.getName(), LockService.Operation.UNLOCK),
                        out.defaultAction
                    });
                    out.sortKey = "2"+ agent.getName();
                    break;
                case ATTACHED:
                    out.icon = ICON_LOCKED_GREEN;
                    out.tooltip = "Locked, attached, double-click to detach";
                    out.defaultAction = getAction(agent.getName(), LockService.Operation.DETACH);
                    out.availableActions = Arrays.asList(new Action[]{
                        getAction(agent.getName(), LockService.Operation.UNLOCK),
                        out.defaultAction
                    });
                    out.sortKey = "1"+ agent.getName();
                    break;
            }
            if (!out.enabled) {
                out.tooltip = "Waiting for response from Lock Manager";
                out.defaultAction = null;
                out.availableActions = null;
            }
        }        
        return out;
    }
    
    
    static protected class RenderedLock implements Comparable<RenderedLock> {
        
        public ImageIcon icon;
        public String tooltip;
        public boolean enabled;
        public Action defaultAction;
        public List<Action> availableActions;
        
        private String sortKey = "";
        
        public RenderedLock(ImageIcon icon, String tooltip, boolean enabled, Action defaultAction, List<Action> availableActions) {
            this.icon = icon;
            this.tooltip = tooltip;
            this.enabled = enabled;
            this.defaultAction = defaultAction;
            this.availableActions = availableActions;
        }
        public RenderedLock() {
        }

        @Override
        public int compareTo(RenderedLock other) {
            return sortKey.compareTo(other.sortKey);
        }
    }

// -- Savin/restoring : --------------------------------------------------------

    @Override
    abstract public Descriptor save();
    
    static public class Descriptor implements Serializable {

        private String name;
        private String agent;
        private AgentPanel.Descriptor agentPanel;
        private DataPanelDescriptor page;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getAgent() {
            return agent;
        }

        public void setAgent(String agent) {
            this.agent = agent;
        }

        public AgentPanel.Descriptor getAgentPanel() {
            return agentPanel;
        }

        public void setAgentPanel(AgentPanel.Descriptor agentPanel) {
            this.agentPanel = agentPanel;
        }

        public DataPanelDescriptor getPage() {
            return page;
        }

        public void setPage(DataPanelDescriptor page) {
            this.page = page;
        }

    }
    
}
